@@ -16,10 +16,21 @@
 #                                                           #
 #   Version 0.2-2                                           #
 #                                                           #
+#   Modified by Hiroyoshi Arai                              #
+#   Date: October, 19, 2010                                 #
+#   Added arguments                                         #
+#     upper: if TRUE, upper-closed (lower-open) intervals.  #
+#     radii.scale: "sqrt"(default) or "linear"              #
+#     border: the color to draw the border.                 #
+#   Revised argument                                        #
+#     col: the color to filling the sector.                 #
+#                                                           #
 #############################################################
 
-rose.diag <- function(x, pch = 16, cex=1, axes = TRUE, shrink = 1, bins=NULL, ticks = TRUE, tcl=0.025, tcl.text=0.125, col=NULL, tol = 0.04, uin=NULL, xlim=c(-1, 1), ylim=c(-1, 1), prop = 1, digits=2, plot.info=NULL, units=NULL, template=NULL, zero=NULL, rotation=NULL, main="", xlab="", ylab="", add=FALSE, ...) {
-  
+rose.diag <- function(x, pch = 16, cex=1, axes = TRUE, shrink = 1, bins=NULL, upper=TRUE, ticks = TRUE, tcl=0.025, tcl.text=0.125, radii.scale = c("sqrt", "linear"), border=NULL, col=NULL, tol = 0.04, uin=NULL, xlim=c(-1, 1), ylim=c(-1, 1), prop = 1, digits=2, plot.info=NULL, units=NULL, template=NULL, zero=NULL, rotation=NULL, main="", xlab="", ylab="", add=FALSE, ...) {
+
+   radii.scale <- match.arg(radii.scale)
+   
    if (is.matrix(x) | is.data.frame(x)) {
       nseries <- ncol(x)
    } else {
@@ -67,11 +78,11 @@ rose.diag <- function(x, pch = 16, cex=1, axes = TRUE, shrink = 1, bins=NULL, ti
          stop("bins must be non negative")
    }
 
-   if (is.null(col)) {
-      col <- seq(nseries)
+   if (is.null(border)) {
+      border <- seq(nseries)
    } else {
-      if (length(col)!=nseries) {
-         col <- rep(col, nseries)[1:nseries]
+      if (length(border)!=nseries) {
+         border <- rep(border, nseries)[1:nseries]
       }
    }
    pch <- rep(pch, nseries, length.out=nseries)
@@ -95,38 +106,48 @@ rose.diag <- function(x, pch = 16, cex=1, axes = TRUE, shrink = 1, bins=NULL, ti
       if (n) {
          x <- conversion.circular(x, units="radians", modulo=modulo)
          attr(x, "circularp") <- attr(x, "class") <- NULL
-         if (rotation=="clock")
-            x <- -x
-         x <- x+zero
+        # x <- x+zero
          if (template=="clock12")
            x <- 2*x
          x <- x%%(2*pi)
-         RosediagRad(x, bins, prop, col[iseries], ...)
+         RosediagRad(x, zero=zero, rotation, bins, upper, radii.scale, prop, border[iseries], col, ...)
       }
    }
    return(invisible(list(zero=zero, rotation=rotation, next.points=0)))    
 }
 
-RosediagRad <- function(x, bins, prop, col, ...) {
+RosediagRad <- function(x, zero, rotation, bins, upper, radii.scale, prop, border, col, ...) {
 #### x musts be in modulo 2pi
     n <- length(x)
     freq <- rep(0, bins)
     arc <- (2 * pi)/bins
+    if (!is.logical(upper))
+       stop("upper must be logical")
+    if (upper == TRUE)
+       x[x == 0] <- 2*pi
+    
     x[x >= 2*pi] <- 2*pi-4*.Machine$double.eps
 #    for (i in 1:bins) {
 #       freq[i] <- sum(x < i * arc & x >= (i - 1) * arc)
 #    }
     breaks <- seq(0,2*pi,length.out=(bins+1))
-    freq <- hist.default(x, breaks=breaks, plot=FALSE, right=TRUE)$counts   
+    freq <- hist.default(x, breaks=breaks, plot=FALSE, right=upper)$counts   
     rel.freq <- freq/n
-    radius <- sqrt(rel.freq) * prop
+    if (rotation == "clock")
+       rel.freq <- rev(rel.freq)
+    
+    if (radii.scale == "sqrt") {
+       radius <- sqrt(rel.freq)*prop
+    } else {
+       radius <- rel.freq*prop
+    }
     sector <- seq(0, 2 * pi - (2 * pi)/bins, length = bins)
     mids <- seq(arc/2, 2 * pi - pi/bins, length = bins)
     for (i in 1:bins) {
        if (rel.freq[i] != 0) {
-          lines.default(c(0, radius[i] * cos(sector[i])), c(0, radius[i] * sin(sector[i])), col=col, ...)
-          lines.default(c(0, radius[i] * cos(sector[i] + (2 * pi)/bins)), c(0, radius[i] * sin(sector[i] + (2 * pi)/bins)), col=col, ...)
-          lines.default(c(radius[i] * cos(sector[i]), radius[i] * cos(sector[i] + (2 * pi)/bins)), c(radius[i] * sin(sector[i]), radius[i] * sin(sector[i] + (2 * pi)/bins)), col=col, ...)
+          xx <- c(0, radius[i]*cos(seq(sector[i], sector[i]+(2*pi)/bins, length=1000/bins)+zero), 0)
+          yy <- c(0, radius[i]*sin(seq(sector[i], sector[i]+(2*pi)/bins, length=1000/bins)+zero), 0)
+          polygon(xx, yy, border=border, col=col, ...)
        }
     }
 }
